
//===============================================================================
//
// coda_et.h
// 
// 8/26/01  D. Lawrence
// 
// The routines in this file can be used to connect to an event transfer or ET
// system and read in CODA formmatted events.
// 
// This is derived from a small piece of code originally given to me by Elliot
// Wolin. His original comment follows:
// 
// 
// 
// //  et_event
// //  gets events from et system
// // to link:
// //    CC -o et_event et_event.cc -I$CODA/common/include -L$CODA/SunOS/lib/ -let -lposix4
// //  ejw, 12-nov-99
//  
//  
// 
//===============================================================================



#include <coda_et.h>


// misc variables
static char session[256]      = "";
static int done               = 0;
int et_ok              = 0;


// et stuff
char *et_station_name        = "C2HIST";
static et_openconfig openconfig;
static et_sys_id et_system_id;
static char et_filename[128];
static et_stat_id et_station_id;
static et_statconfig et_station_config;
static et_att_id et_attach_id;
static et_event *et_event_ptr;
static int et_control[ET_STATION_SELECT_INTS];


// prototypes
void connect_et();



//--------------------------------------------------------------------------


int init_et(char *session_name) {
	char *ptr;
  
	cout<<"Initializing ET...\n";

	// synch with c i/o
	ios::sync_with_stdio();
  
	// get session name
	if(ptr=getenv("SESSION"))strcpy(session,ptr);
	if(session_name)strcpy(session,session_name);
	if(!strlen(session))strcpy(session,"parasitic");

	cout<<"ET session name: "<<session<<"\n";

	et_ok=0;

	// create et file name
	sprintf(et_filename,"/tmp/et_sys_%s",session);
	et_open_config_init(&openconfig);

	// create station config in case no station exists
	et_station_config_init(&et_station_config);
	et_station_config_setblock(et_station_config,ET_STATION_NONBLOCKING);
	et_station_config_setselect(et_station_config,ET_STATION_SELECT_ALL);
	et_station_config_setuser(et_station_config,ET_STATION_USER_MULTI);
	et_station_config_setrestore(et_station_config,ET_STATION_RESTORE_OUT);
	et_station_config_setcue(et_station_config,100);
	et_station_config_setprescale(et_station_config,1);
	cout<<"ET station configured\n";

	// connect to the ET system
	if(et_ok==0) {
		connect_et();
		if(et_ok==0){
			sleep(1);
		}else{
			cout<<"et_ok="<<et_ok<<"\n";
		}
	}

	return et_ok;
}

//--------------------------------------------------------------------------



void end_et(void){
  
	// done
	et_close(et_system_id);
}


//--------------------------------------------------------------------------


int ReadEventFromET(CODA_header_t **coda_evt)
{

#define buff_size 8192  /* maximum event size in bytes */

	static char buff[buff_size];
	long *evt;
	int err=0;

    // check et system...die if et goes away
    if(et_alive(et_system_id)==0) return -1;
    
    
    // et system ok...get event
    if(et_event_get(et_system_id,et_attach_id,&et_event_ptr,ET_SLEEP,NULL)!=ET_OK) {
      cerr << "?error return from et_event_get" << endl;
      return -2;
    }

    // get control words and pointer to event data
    et_event_getcontrol(et_event_ptr,et_control);
    et_event_getdata(et_event_ptr,(void**)&evt);

    // Copy event into local buffer
    if(et_event_ptr->length>buff_size){
    	cerr<<__FILE__<<":"<<__LINE__<<" event size too big for buffer!"<<endl;
    	err = -1;
    }else{
		memcpy(buff,(char*)evt,et_event_ptr->length);
		*coda_evt = (CODA_header_t*)buff;
	}
    
    // put event back
    et_event_put(et_system_id,et_attach_id,et_event_ptr);


    return err;
}


//-------------------------------------------------------------------


void connect_et() {
  
  int status;
  sigset_t sigblock;
  

  et_ok=1;


  // open et system
  cout<<"ET filename: "<<et_filename<<"\n";
  if(et_open(&et_system_id,et_filename,openconfig)!=ET_OK)return;
  cout<<"ET system opened.\n";


  // create station if not already created
  status=et_station_create(et_system_id,&et_station_id,et_station_name,et_station_config);
  if((status!=ET_OK)&&(status!=ET_ERROR_EXISTS)) { 
    et_close(et_system_id);
    cerr << "Unable to create station " << et_station_name << endl;
    done=1;
    return;
  }
  cout<<"ET station created\n";


  // block signals to THIS thread and any thread created by this thread
  // needed to keep signals from et threads
  sigfillset(&sigblock);
  pthread_sigmask(SIG_BLOCK,&sigblock,NULL);


  // attach to station
  status=et_station_attach(et_system_id,et_station_id,&et_attach_id);
  if(status!=ET_OK) {
    et_close(et_system_id);
    cerr << "Unable to attach to station " << et_station_name << endl;
    done=1;
    return;
  }
  

  // unblock signals
  pthread_sigmask(SIG_UNBLOCK,&sigblock,NULL);


  // success
  et_ok=0; 
  cout << "...now connected to ET system: " << et_filename 
       << ",   station: " << et_station_name << endl;

  return;
}


//--------------------------------------------------------------------------

void primex_et_wait_for_alive(void)
{
	if (!et_alive(et_system_id))et_wait_for_alive(et_system_id);
}


