
//===============================================================================
//
// coda_parse.cc
// 
// 8/16/01  D. Lawrence
// 
// The routines in this file can be used to parse CODA formatted data.
// There are also routines which can parse the data within the CODA
// bank that is in the format presented by the digitization electronics.
// Specifically, the LRS1881M ADC and LRS1877 TDC.
// 
//===============================================================================


#include <coda_parse.h>


//=======================================================================================
// This routine will look through the data pointed to by "data" and fill out pointers
// to the various banks contained therein. 
//
//  *event_id     Pointer to an event_id data structure.
//
//   **roc_data   Pointer to an array of pointers of CODA_roc_data_t. This array will be
//                filled with pointers to the data in the event. 
// 
//   *Nptrs       On entry, this gives the number of elements in the array of roc_data
//                pointers. On exit, it contains the number of valid roc_data pointers.
//
//
//  Return Value  If the outer bank is a non-physics event type, the negative of the
//                event type will be returned.
//                If the outer bank is a physics event type, the data is searched for
//                RunControl or Sync event banks. If any are found, that type is returned.
//                If the outer bank is a physics event type and there are no RunControl
//                or Sync event bank banks (i.e. only ROC banks), then zero is returned.
//                In this way, the return value can be used to as a simple flag to determine
//                if ROC data exists (return zero) or not (return non-zero).
//  
//  
//=======================================================================================
int GetBankPtrs(char *data, CODA_event_id_t **event_id, CODA_roc_data_t **roc_data, int *Nptrs )
{
	CODA_header_t *head;
	int Nroc_banks;
	int max_roc_ptrs = *Nptrs;
	int n_words;
	int *iptr;
	int i;
	static int last_event_numnber=0;

	// Initialize pointers
	*event_id = NULL;
	*Nptrs = 0;

	// Check for valid PHYSICS event type
	head = (CODA_header_t*)data;
	if(head->bank.data_type != CODA_EVENT_TYPE_PHYSICS)return -head->bank.data_type;

	// First bank inside physics event should be event ID bank
	// Assume this is the case, but check just to make sure.
	*event_id = (CODA_event_id_t*)&head->data_bank;
	if((*event_id)->bank.tag != 0xC000){
		cerr<<__FILE__<<":"<<__LINE__<<" Bad event_id bank!"<<endl;
		cerr<<"\t length    = "<<(*event_id)->bank.length		<<endl;
		cerr<<"\t num       = "<<(int)(*event_id)->bank.num			<<endl;
		cerr<<"\t data_type = "<<(int)(*event_id)->bank.data_type	<<endl;
		cerr<<"\t tag       = 0x"<<hex<<(*event_id)->bank.tag	<<dec<<endl;
		cerr<<"\t previous event = "<<last_event_numnber	<<endl;

		return 999; // This isn't terribly elegant, but we can't very well continue.
	}
	last_event_numnber = (*event_id)->event_number;

	// Now, loop over the remaining banks placing pointers to them in the
	// roc_data array
	Nroc_banks = 0;
	n_words = (head->bank.length-1) - 5; // -1 for info in physics header, -5 for event_id bank
	iptr = &(*event_id)->next_bank;
	while(n_words>0){

		roc_data[Nroc_banks] = (CODA_roc_data_t*)iptr;
		iptr += roc_data[Nroc_banks]->length + 1;
		
		n_words -= roc_data[Nroc_banks++]->length+1;
		if(Nroc_banks >=max_roc_ptrs)break;
	}
	*Nptrs = Nroc_banks;

	// At this point, we can check if any of the banks we found is a RunControl
	// or Sync event. Return the event type if so.
	for(i=0;i<Nroc_banks;i++){
		if((roc_data[i]->counter==0xCC) && (roc_data[i]->type==1))
			return ((CODA_bank_t*)&roc_data[i])->tag;
	}

	// If we got here, it must be ROC data. return 0;
	return 0;
}



//=======================================================================================
// Look through a ROC data bank for data from the given slot number.
// Assume it is from an LRS1881M and copy the values into the adc array.
//=======================================================================================
int ParseLRS1881Mdata(CODA_roc_data_t *roc_data,int slot,unsigned int adc[64])
{
	int i;
	unsigned int head;
	unsigned int word_count,buffer_number,bn,word_parity,address,channel;
	unsigned int val;
	unsigned int *data=NULL;

	// Initialize array
	for(i=0;i<64;i++)adc[i]=0;

	// First, Check that the start and end flags are present
	if((roc_data->data&0xda000000) != 0xda000000)return -1;
	if(((&roc_data->data)[roc_data->length-2]&0xda000000) != 0xda000000)return -2;

	// Loop over all words, looking for the given slot number. Break at the
	// first occurance since that should be the header word. The data immediately
	// following the header word should be the data words.
	for(i=1;i<=roc_data->length-3;i++){
		head = (&roc_data->data)[i];
		address = (head>>27)&0x1F;
		if(address == slot){
			data=&(&roc_data->data)[i];
			break;
		}
	}
	if(data==NULL)return -3;

	// OK, we found data for this slot. Get additional info from the header
	word_count=head&0x7F;
	buffer_number=(head>>7)&0x3F;
	word_parity=(head>>15)&0x01;

	// Loop over all data words. "data" should actually be pointing to the header
	// word. The word count includes the header though so we index starting from 1
	// and go up to word_count-1;
	for(i=1;i<word_count;i++){
		bn			=(data[i]>>24)&0x03;
		word_parity	=(data[i]>>15)&0x01;
		address		=(data[i]>>27)&0x1F;
		channel 	=(data[i]>>17)&0x7F;
		val			=(data[i]    )&0x3FFF;
		
		if(channel>=0 && channel<=63)adc[channel]=val;
	}
}


//=======================================================================================
// Look through a ROC data bank for data from the given slot number.
// Assume it is from an LRS1877 and copy the values into the tdc array.
//=======================================================================================
int ParseLRS1877data(CODA_roc_data_t *roc_data,int slot,TDC1877_t *tdc, int *Nhits)
{
	int i;
	unsigned int head;
	unsigned int word_count,buffer_number,bn,word_parity,address,channel;
	unsigned int edge;
	unsigned int val;
	unsigned int *data=NULL;
	int max_tdc_hits = *Nhits;

	// Initialize number of hits
	*Nhits = 0;

	// First, Check that the start and end flags are present
	if((roc_data->data&0xda000000) != 0xda000000)return -1;
	if(((&roc_data->data)[roc_data->length-2]&0xda000000) != 0xda000000)return -2;

	// Loop over all words, looking for the given slot number. Break at the
	// first occurance since that should be the header word. The data immediately
	// following the header word should be the data words.
	for(i=1;i<=roc_data->length-3;i++){
		head = (&roc_data->data)[i];
		address = (head>>27)&0x1F;
		if(address == slot){
			data=&(&roc_data->data)[i];
			break;
		}
	}
	if(data==NULL)return -3;

	// OK, we found data for this slot. Get additional info from the header
	word_count=head&0x07FF;
	buffer_number=(head>>11)&0x07;
	word_parity=(head>>15)&0x01;

	// Loop over all data words. "data" should actually be pointing to the header
	// word. The word count includes the header though so we index starting from 1
	// and go up to word_count-1;
	for(i=1;i<word_count;i++){
		bn			=(data[i]>>24)&0x03;
		word_parity	=(data[i]>>26)&0x01;
		address		=(data[i]>>27)&0x1F;
		channel 	=(data[i]>>17)&0x7F;
		edge		=(data[i]>>16)&0x01;
		val			=(data[i]    )&0xFFFF;

		if(*Nhits>=max_tdc_hits)return -4;
		
		tdc[*Nhits].slot	= address;
		tdc[*Nhits].id		= channel;
		tdc[*Nhits].edge	= edge;
		tdc[*Nhits].val		= val;

		*Nhits += 1;
	}

	return 0;
}

//=======================================================================================
// This routine will call one of the above parser routines for every
// bit set in the LRS1881M_slots and LRS1877_slots masks. This makes
// things a little more convienient.
//=======================================================================================
int ParseAlldata(CODA_roc_data_t *roc_data
	,unsigned int LRS1881M_slots, unsigned int adc[64]
	,unsigned int LRS1877_slots, TDC1877_t *tdc, int *Nhits)
{
	int slot;
	int n_tdc_hits;
	int max_tdc_hits = *Nhits;

	// Loop over all slots
	*Nhits = 0;
	for(slot=0;slot<=25;slot++){
	
		if((LRS1881M_slots>>slot) && 0x01)ParseLRS1881Mdata(roc_data, slot, adc);

		if((LRS1877_slots >>slot) && 0x01){
			n_tdc_hits = max_tdc_hits - *Nhits;
			ParseLRS1877data(roc_data, slot, &tdc[*Nhits], &n_tdc_hits);
			*Nhits += n_tdc_hits;
		}
	
	}

	return 0;
}





